/*
 * makeAutomaton(initialState)
 * 
 * Given an initialState, in the form of a two dimensional array,
 * that represents a bit vector in little endian format (where the least
 * significant bit is at the left end of the vector), repeatedly
 * increment the vector.
 *
 * This code implements a simulation of the counter of John Simon's 
 * "Every Icon" artwork.
 *
 * Warren Sack <wsack@ucsc.edu>
 * May 2017
 * 
 */

function makeAutomaton(initialState) {

  ////
  //// State information
  ////

  //
  // copy2DArray(a): array
  // 
  // Given a 2D array, make a copy of each of its values
  // a return the copy as a 2D array.
  //
  function copy2DArray(a) {
    var c = [];
    for ( var i = 0; i < a.length; i++ ) {
      for ( var j = 0; j < a[i].length; j++ ) {
	c[i] = a[i].slice(0);
      }
    }
    return(c);
  }

  // initialState contains the first state of the automaton
  // currentState is the current state of the automaton, written as a 2D array
  var currentState = copy2DArray(initialState);

  // 
  // getCurrentState(): array
  //
  // Returns the 2D array that represents the current state of the bit vector.
  // This is useful for saving a state.
  //
  function getCurrentState() {
    return(currentState);
  }

  // 
  // setCurrentState(state): array
  //
  // Replaces the 2D array that represents the current state of the bit vector.
  // This is useful for starting the counter in a specific state.
  //
  function setCurrentState(state) {
    currentState = copy2DArray(state);
    return(currentState);
  }

  // arrayWidth defines the width of the 2D array that holds the bit vector.
  var arrayWidth = initialState[0].length;

  // arrayHeight defines the height of the 2D array that holds the bit vector.
  var arrayHeight = initialState.length;

  //
  // getValueAtPosition(p): value
  //
  // Given a position in the array representing the current state of the automaton,
  // returns the value in the array of that position.  The position, p, is an ordinal, a
  // single integer, but the number is stored in a two dimensional array.
  //
  function getValueAtPosition(p) {
    var x = p % arrayWidth;
    var y = Math.floor( p / arrayWidth );
    return(currentState[x][y]);
  }

  //
  // setValueAtPosition(p,v): value
  //
  // Given a position in a one dimensional array, convert it to a position in a 
  // two dimensional array.  Set the value in the 2D array to v.
  //
  function setValueAtPosition(p,v) {
    var x = p % arrayWidth;
    var y = Math.floor( p / arrayWidth );
    currentState[x][y] = v;
    return(currentState[x][y]);
  }

  ////
  //// Evaluation
  ////

  //
  // littleEndianBitIncrement(): side effects currentState
  //
  // Increment the value of the least significant bit in the array
  // representing the currentState.  The array is in little endian
  // format meaning that the least significant bit is on the left
  // rather than on the right side of the bit vector.
  //
  function littleEndianBitIncrement() {
    var carryBit = 1;
    var currentBit = 0;
    for ( var p = 0; p < ( arrayWidth * arrayHeight ); p++ ) {
      currentBit = carryBit + (1 * getValueAtPosition(p));
      if ( currentBit === 2 ) {
	carryBit = 1;
	setValueAtPosition(p,"0");
      }
      else {
	setValueAtPosition(p,"" + currentBit);	  
	break;
      }
    }
  }

  //
  // computeNextState(): side-effects currentState
  //
  function computeNextState() {
    littleEndianBitIncrement();
  }

  //
  // runAutomaton()
  //
  // Run the automaton.
  //
  function runAutomaton() {
    computeNextState();
  }

  return({
        arrayWidth : arrayWidth,
	arrayHeight : arrayHeight,
        getCurrentState : getCurrentState,
        setCurrentState : setCurrentState,
	computeNextState : computeNextState,
	runAutomaton : runAutomaton,
        getValueAtPosition : getValueAtPosition,
        setValueAtPosition : setValueAtPosition});

}






