/*
 * makeDisplay(automaton,canvasName)
 *
 * This code implements a simulation of the display of John Simon's "Every Icon" artwork:
 * a square matrix of smaller black and white squares representing a large bit vector in
 * big endian format repeatedly incremented.
 *
 * Press <space> to turn the counter on or off.
 * Press <p> to take a snapshot (a PNG) of the current display.
 * Press <mouse> to turn a square/bit on or off.
 *
 * Warren Sack <wsack@ucsc.edu>
 * May 2017
 * 
 */

function makeDisplay(automaton,canvasName) {

  window.resizeTo(screen.width,screen.height);
  var width = automaton.arrayWidth;
  var height = automaton.arrayHeight;
  var lineWidth = 1;
  var margin = screen.availTop;
  var squareWidth = ( screen.width - ( 2 * margin ) ) / width;
  var squareHeight = ( screen.height - ( 2 * margin ) ) / ( 1.3 * height );
  if ( squareWidth > squareHeight ) { squareWidth = squareHeight; }
  var canvas = document.getElementById(canvasName);
  canvas.width = ( squareWidth * width ) + ( 2 * margin );
  canvas.height = ( squareWidth * height ) + ( 2 * margin );
  window.addEventListener('mousedown',recolorSquare,false);
  window.addEventListener('keydown',startStop,false);
  window.addEventListener('keydown',snapShot,false);
  var context = canvas.getContext('2d');
  var running = false;
  var timeBetweenFrames = 500;  // reduce this number to speed the counter up
  var intervalID = setInterval(updateAndDisplay,timeBetweenFrames);
  if (! running) { clearInterval(intervalID); }


  //
  // updateAndDisplay()
  //
  // Increment the counter and update the display.
  //
  function updateAndDisplay() {
    automaton.computeNextState(); 
    drawMatrix();
   }

  //
  // startStop(event)
  //
  // Start or stop the counter.
  //
  function startStop(event) {
    if (event.keyCode == 32) {
      if ( running ) { 
	clearInterval(intervalID); 
	running = false;
      }
      else {
	intervalID = setInterval(updateAndDisplay,timeBetweenFrames);
	running = true;
      }
    }
  }

  //
  // snapShot(event)
  //
  // Take a snapshot of the current state of the counter.
  // Save it as a PNG.
  //
  function snapShot(event) {
    if (event.keyCode == 80) {
       image = new Image();
       image = canvas.toDataURL("image/png");
       window.open(image,"snapShot");
       window.focus();
    }
  }
  
  //
  // recolorSquare(event)
  //
  // Record where the mousedown event occurred, then recolor the
  // square below the mouse.  If the square was white, recolor it
  // black; and, vice versa.  Record the color change as a change in
  // the value of the counter.
  //
  function recolorSquare(event) {
    var xPos = event.clientX;
    var yPos = event.clientY;
    // translate this to an [column][row] position for the matrix
    var column = -1;
    var row = -1;
    var x = margin;
    for ( var c = 0; c < width; c++ ) {
      x = x + squareWidth;
      if ( x > xPos ) { column = c; break; }
    }
    var y = margin;
    for ( var r = 0; r < height; r++ ) {
      y = y + squareWidth;
      if ( y > yPos ) { row = r; break; }
    }
    // Set the square to "1" if it is "0" and vice versa.
    // And, recolor the square appropriately (black to white;
    // or, white to black).
    if ( ( row !== -1 ) && ( column !== -1 ) ) {
      x = x - squareWidth;
      y = y - squareWidth;
      context.strokeRect(x,y,squareWidth,squareWidth);
      if ( automaton.getValueAtPosition((row*width)+column) === "1" ) {
	automaton.setValueAtPosition(((row*width)+column),"0");
	context.fillStyle = 'white';
      }
      else { 
	automaton.setValueAtPosition(((row*width)+column),"1");
	context.fillStyle = 'black';
      }
      context.fillRect((x + lineWidth),
		       (y + lineWidth),
		       (squareWidth - (2 * lineWidth)),
		       (squareWidth - (2 * lineWidth)));
    }
  }

  //
  // drawMatrix()
  // 
  // Iterate through the current state of the counter, a bit vector.
  // For each position in the vector that contains a "1", color a
  // square on the display black.  For each that contains a "0", color
  // a square white on the display.
  //
  function drawMatrix() {
    var x = margin;
    var y = margin;
    context.lineWidth = lineWidth;
    context.strokeStyle = 'gray';
    for ( var row = 0; row < height; row++ ) {
      for ( var column = 0; column < width; column++ ) {
       	context.strokeRect(x,y,squareWidth,squareWidth);
	if ( automaton.getValueAtPosition((row*width)+column) === "1" ) {
	  context.fillStyle = 'black';
	}
	else {
	  context.fillStyle = 'white';
	}
	context.fillRect((x + lineWidth),
			 (y + lineWidth),
			 (squareWidth - (2 * lineWidth)),
			 (squareWidth - (2 * lineWidth)));
	x = x + squareWidth;
      }
      x = margin;
      y = y + squareWidth;
    }
  }

  return({
        drawMatrix : drawMatrix,
	updateAndDisplay : updateAndDisplay});
 
}
